# Licensed to Elasticsearch B.V under one or more agreements.
# Elasticsearch B.V licenses this file to you under the Apache 2.0 License.
# See the LICENSE file in the project root for more information

__title__ = "eland"
__description__ = "Python Client and Toolkit for DataFrames, Big Data, Machine Learning and ETL in Elasticsearch"
__url__ = "https://github.com/elastic/eland"
__version__ = "7.7.0a1"
__author__ = "Steve Dodson"
__author_email__ = "steve.dodson@elastic.co"
__maintainer__ = "Seth Michael Larson"
__maintainer_email__ = "seth.larson@elastic.co"
